package javato.activetesting.activechecker;

import javato.activetesting.common.MersenneTwisterFast;
import javato.activetesting.common.WeakIdentityHashMap;
import javato.activetesting.common.Parameters;
import javato.activetesting.scheduler.StallBreaker;

import javato.activetesting.coverage.* ;
import javato.activetesting.SPTester ;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.Iterator ;

/**
 * Copyright (c) 2007-2008,
 * Koushik Sen    <ksen@cs.berkeley.edu>
 * All rights reserved.
 * <p/>
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * <p/>
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * <p/>
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * <p/>
 * 3. The names of the contributors may not be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 * <p/>
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
public class ActiveChecker {
	static public LinkedList<ActiveChecker> blockedThreads = new LinkedList<ActiveChecker>();
	static private Map threadToSemaphoreAndTime = Collections.synchronizedMap(new WeakIdentityHashMap());
	final public static Object lock = new Object();
	public static MersenneTwisterFast rand = new MersenneTwisterFast(System.currentTimeMillis());
	public final static AtomicBoolean dirty = new AtomicBoolean(false);
	//    private static double probability = Parameters.probability;
	//    private static int desc = Parameters.desc;
	private static double probability = 1.0 ;
	private static double pb_control = Parameters.feedback_probability ;


	static public int getBlockedThreadCount() {
		return blockedThreads.size();
	}

	public static void reduceProbability() {
//		probability /= 2;
//		System.out.println("Probability "+probability);
	}

	class Pair {
		int waitTime;
		Semaphore sem;
		ActiveChecker checker;

		Pair(int waitTime, Semaphore sem, ActiveChecker checker) {
			this.waitTime = waitTime;
			this.sem = sem;
			this.checker = checker;
		}
	}


	private Semaphore sem;

	final protected void block(int milliSeconds) {
		if (milliSeconds > 0) {
			threadToSemaphoreAndTime.put(Thread.currentThread(), new Pair(milliSeconds, null, this));
		} else {
			sem = new Semaphore();
			blockedThreads.add(this);
			dirty.set(true);
			threadToSemaphoreAndTime.put(Thread.currentThread(), new Pair(0, sem, this));
		}
	}

	final protected void unblock(int milliSeconds) {
		synchronized (lock) {
			//System.out.println("Before "+blockedThreads.size());
			blockedThreads.remove(this);
			dirty.set(true);
			//System.out.println("After "+blockedThreads.size());
			sem.release(milliSeconds);
		}
	}

	final public static void blockIfRequired() {
		Pair p = (Pair) threadToSemaphoreAndTime.remove(Thread.currentThread());
		if (p != null) {
			if (p.waitTime > 0) {
				try {
					Thread.sleep(p.waitTime);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			} else {
				StallBreaker.breakAnyStall();
				try {
					p.sem.acquire();
					//						return false ;
					//return true ;
				} catch (InterruptedException e) {
					blockedThreads.remove(p.checker);
					dirty.set(true);
				}
			}
		}
		//return false ;
	}

	public void check(Collection<ActiveChecker> checkers) {
		block(0);
	}

	final public void check() {
		synchronized (lock) {
			if (rand.nextDouble()<=probability) {
				int activeThreadCount = StallBreaker.getActiveThreadCount();
				if (activeThreadCount > 1 && threadToSemaphoreAndTime.get(Thread.currentThread()) == null) {
					check(blockedThreads);
				}
			}
		}
	}

	final public void check(int waitTime) {
		synchronized (lock) {
			int activeThreadCount = StallBreaker.getActiveThreadCount();
			if (activeThreadCount > 1 && threadToSemaphoreAndTime.get(Thread.currentThread()) == null) {
				block(waitTime);
			}
		}
	}

	public static void Check() {
		(new ActiveChecker()).check();
		blockIfRequired();
	}

	public void unblockControl() {
		synchronized (lock) {
			//System.out.println("(log) unblockControl") ;
			blockedThreads.remove(this);
			dirty.set(true);
			//System.out.println("After "+blockedThreads.size());
			sem.release(0);
		}
	}

	public static boolean unblockAThread() {
		synchronized (lock) {
			if (blockedThreads.size() == 0) return false;
			//System.out.println("(log) unblockAThread") ;

			if (blockedThreads.getFirst() instanceof CovActiveChecker) {
				if (rand.nextDouble() <= pb_control ) {
					Iterator<ActiveChecker> itr = blockedThreads.iterator() ;
					
					LinkedList<CovActiveChecker> candidates = new LinkedList<CovActiveChecker>() ;
					while(itr.hasNext()){
						CovActiveChecker c = (CovActiveChecker) itr.next();
						int lock = c.lock.intValue();
						
						int lastiid=-1;
						if (SPTester.last.get(lock)!=null)
							lastiid = SPTester.last.get(lock).intValue();
						if((c.notyetcovered.contains(new SyncPair(lastiid, c.iid.intValue()))))
							candidates.add(c);
					}
					
					if (candidates.size()>0){ 
						candidates.get(rand.nextInt(candidates.size())).unblock(0);
						return true;
					}
					
					
					itr = blockedThreads.iterator() ;
					LinkedList<CovActiveCheckerPair> candidatePairs = new LinkedList<CovActiveCheckerPair>() ;
					
//					candidates = new LinkedList<CovActiveChecker>();
					
					for (int i=0 ; i<blockedThreads.size() ; i++)
						for (int j=0 ; j<blockedThreads.size() ; j++) {
							CovActiveChecker a = (CovActiveChecker) blockedThreads.get(i);
							CovActiveChecker b = (CovActiveChecker) blockedThreads.get(j);
							if(a.lock.intValue() == b.lock.intValue()){
								if (a.notyetcovered.contains(new SyncPair(a.iid.intValue(), b.iid.intValue())))
									candidates.add(a);
								if (a.notyetcovered.contains(new SyncPair(b.iid.intValue(), a.iid.intValue())))
									candidates.add(b);
							}
						}
					
					if (candidates.size()>0){
						candidates.get(rand.nextInt(candidates.size())).unblock(0);
						return true;
					}
					
					SPTester.countRule3_normal++;
					SPTester.countRule3_max++;
					candidates = new LinkedList<CovActiveChecker>() ;
					itr = blockedThreads.iterator() ;
					int minValue = -1 ;
					int maxValue = -1 ;
					CovActiveChecker minChecker = null ;
					
					while (itr.hasNext() == true) {
						CovActiveChecker c = (CovActiveChecker)itr.next() ;
						if(maxValue == -1 || c.score() > maxValue)
							maxValue = c.score();
						
						if ((minChecker == null) || (c.score() < minValue)) {
							minValue = c.score() ;
							minChecker = c ;
							candidates = new LinkedList<CovActiveChecker>() ;
							candidates.add(c) ;
						}
						else if (c.score()==minValue) {
							candidates.add(c) ;
						}

					}

					if(maxValue == 0)
						SPTester.countRule3_max--;
					
					if (candidates.size() > 0) {
						int choice = rand.nextInt(candidates.size()) ;
						//System.out.println("(log) unblockAThread thread-" + candidates.get(choice).thread  + "(" + minValue + ") among " + candidates.size()) ;	
						candidates.get(choice).unblock(0) ;
						//maxChecker.unblock(0) ;
						return true ;
					}
					else{
						if (blockedThreads.size() > 0) {
							int randNum = rand.nextInt(blockedThreads.size());
							blockedThreads.get(randNum).unblock(0);
							//System.out.println("(log) unblockAThread - random") ;
							return true ;
						}
					}
				}
				else {	
					int randNum = rand.nextInt(blockedThreads.size());
					blockedThreads.get(randNum).unblock(0);
					return true;
				}
			}
			
			synchronized (lock) {
				if (blockedThreads.size() > 0) {
					int randNum = rand.nextInt(blockedThreads.size());
					blockedThreads.get(randNum).unblock(0);

				}
			}
			return true;
		}
	}

	public boolean equals(Object o) {
		return this == o;
	}
}
